
# q-expansion of newform 196.6.e.i, downloaded from the LMFDB on 08 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 196
weight = 6
poly_data = [1, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[99, [1, 0]], [101, [0, -1]]]
aps_data = [[0, 0], [26, -26], [0, 16], [0, 0], [-8, 8], [-684, 0], [-2218, 2218], [0, -2698], [0, -3344], [-3254, 0], [4788, -4788], [0, 11470], [-13350, 0], [-928, 0], [0, 1212], [-13110, 13110], [34702, -34702], [0, -1032], [-10108, 10108], [62720, 0], [-18926, 18926], [0, -11400], [-88958, 0], [0, 19722], [-17062, 0], [45904, -45904], [0, -136012], [0, 69156], [146414, -146414], [-80186, 0], [274800, 0], [0, 180742], [209678, -209678], [-17242, 0], [0, -59358], [336344, -336344], [464588, -464588], [0, -314792], [-285724, 0], [0, -709148], [617148, -617148], [-237828, 0], [0, 133512], [-270446, 270446], [875102, 0], [-347620, 347620], [-425380, 0], [-481592, 0], [-6042, 6042], [0, 1804], [0, 1611526], [-987096, 0], [893510, -893510], [-365946, 0], [0, 1404594], [-1099680, 1099680], [814948, -814948], [0, -1699056], [1365082, -1365082], [-715846, 0], [217726, -217726], [-1507080, 0], [-12502, 0], [-647432, 647432], [0, -935978], [0, -705942], [0, 1143040], [-2364018, 0], [-726240, 726240], [-136180, 0], [-1169070, 1169070], [0, 4280], [2447960, -2447960], [0, 904514], [-4230344, 0], [0, 4553996], [3985406, -3985406], [0, 552420], [0, -38190], [-3924754, 3924754], [-598386, 0], [4615974, 0], [61560, -61560], [3797266, 0], [0, -2288520], [0, 4759756], [-4367150, 0], [0, -5449942], [-1669660, 0], [70768, 0], [0, 5660830], [-1449484, 1449484], [-4075040, 4075040], [986100, 0], [0, -5983420], [3493728, 0], [0, 2157108], [-6658170, 6658170], [0, 5952230], [0, 6396810], [-5518512, 0], [-2021590, 2021590], [8146778, -8146778], [0, 11982426], [-1395816, 1395816], [1967842, -1967842], [3188974, 0], [0, 1675002], [10063464, -10063464], [-1727978, 0], [0, -16952256], [-10194162, 10194162], [15745190, 0], [-332690, 332690], [3597200, 0], [14638930, -14638930], [13838614, 0], [-14035812, 14035812], [0, -16106274], [4800752, 0], [-17656472, 17656472], [6592250, 0], [0, 9771780], [18866316, -18866316], [0, 8670182], [7934818, 0], [0, -26260034], [0, 22076292], [-8492452, 0], [0, -19071288], [14683224, -14683224], [16426488, 0], [0, 24435672], [-295566, 0], [0, -473842], [-23324062, 0], [15558280, -15558280], [6668430, -6668430], [12246108, 0], [29149466, -29149466], [-7588486, 0], [0, 5988490], [-817960, 817960], [2519628, 0], [-16100600, 16100600], [25816732, 0], [15427012, 0], [3605166, -3605166], [0, 4069946], [0, -7251112], [0, -2374138], [-30355818, 0], [1533380, 0], [0, -29237868], [-44822748, 44822748], [-36290640, 0], [0, -32535016], [0, -44667594], [-15668018, 0], [-20117480, 20117480], [0, 8845184], [-31436374, 0], [-9521584, 0], [0, 10652046], [-27293082, 27293082], [10476372, -10476372], [-1882304, 1882304], [27151832, -27151832]]
